import {useContext, useState, useEffect} from 'react';
import {AppContext} from '@/context/AppContext';
import {api, isSsoSupported} from '@/utils';
import {config} from '@/config';
import {useSessionStorage} from '@/hooks';

export function useSso() {
    const {currentServer, currentVersion} = useContext(AppContext);
    const {clearSsoDataStorage} = useSessionStorage();
    const [ssoSupported, setSsoSupported] = useState(false);
    const [loginUrl, setLoginUrl] = useState('');
    const [isLoading, setIsLoading] = useState(false);

    const handleSupportSso = (serverSupportsSso) => {
        const clientSupportsSso = isSsoSupported(currentVersion);
        const isSupported = clientSupportsSso && serverSupportsSso;

        if (isSupported) {
            setSsoSupported(true);
        } else {
            clearSsoDataStorage();
            setSsoSupported(false);
        }
    };

    useEffect(() => {
        if (!currentServer) {
            handleSupportSso(false);
            return;
        }

        setIsLoading(true);

        api.get('/api/auth/sso/log_in', { query: { target_server: currentServer } })
            .then((result) => {
                try {
                    const url = new URL(result.loginUrl);
                    const params = new URLSearchParams(url.search);

                    const originalRedirectUri = params.get('redirect_uri');
                    if (originalRedirectUri) {
                        const redirectUrl = new URL(originalRedirectUri);
                        redirectUrl.searchParams.set('target_server', currentServer);
                        params.set('redirect_uri', config.redirectUri);
                    }

                    url.search = params.toString();
                    handleSupportSso(true);
                    setLoginUrl(url.toString());
                } catch (error) {
                    console.error('Invalid login URL:', error);
                    handleSupportSso(false);
                    setLoginUrl('');
                }
            })
            .catch((error) => {
                console.error('Failed to fetch SSO login URL:', error);
                handleSupportSso(false);
                setLoginUrl('');
            })
            .finally(() => {
                setIsLoading(false);
            });
    }, [currentServer, currentVersion]);

    return {
        loginUrl,
        isLoading,
        ssoSupported,
    };
}
